@@ -1,30 +1,10 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "cmakeconfigitem.h"
 
+#include "cmakeprojectmanagertr.h"
+
 #include <projectexplorer/kit.h>
 
 #include <utils/algorithm.h>
@@ -188,7 +168,7 @@ QString CMakeConfigItem::typeToTypeString(const CMakeConfigItem::Type t)
     return {};
 }
 
-Utils::optional<bool> CMakeConfigItem::toBool(const QString &value)
+std::optional<bool> CMakeConfigItem::toBool(const QString &value)
 {
     // Taken from CMakes if(<constant>) documentation:
     // "Named boolean constants are case-insensitive."
@@ -214,7 +194,17 @@ QString CMakeConfigItem::expandedValue(const ProjectExplorer::Kit *k) const
 
 QString CMakeConfigItem::expandedValue(const Utils::MacroExpander *expander) const
 {
-    return expander ? expander->expand(QString::fromUtf8(value)) : QString::fromUtf8(value);
+    QString expandedValue = expander ? expander->expand(QString::fromUtf8(value))
+                                     : QString::fromUtf8(value);
+
+    // Make sure we have CMake paths using / instead of \\ on Windows
+    // %{buildDir} returns \\ on Windows
+    if (type == CMakeConfigItem::FILEPATH || type == CMakeConfigItem::PATH) {
+        const FilePaths paths = transform(expandedValue.split(";"), &FilePath::fromUserInput);
+        expandedValue = transform(paths, &FilePath::path).join(";");
+    }
+
+    return expandedValue;
 }
 
 bool CMakeConfigItem::less(const CMakeConfigItem &a, const CMakeConfigItem &b)
@@ -225,13 +215,13 @@ bool CMakeConfigItem::less(const CMakeConfigItem &a, const CMakeConfigItem &b)
 CMakeConfigItem CMakeConfigItem::fromString(const QString &s)
 {
     // Strip comments (only at start of line!):
-    int commentStart = s.count();
-    for (int i = 0; i < s.count(); ++i) {
+    int commentStart = s.size();
+    for (int i = 0; i < s.size(); ++i) {
         const QChar c = s.at(i);
         if (c == ' ' || c == '\t')
             continue;
         else if ((c == '#')
-                 || (c == '/' && i < s.count() - 1 && s.at(i + 1) == '/')) {
+                 || (c == '/' && i < s.size() - 1 && s.at(i + 1) == '/')) {
             commentStart = i;
             break;
         } else {
@@ -244,7 +234,7 @@ CMakeConfigItem CMakeConfigItem::fromString(const QString &s)
     int firstPos = -1;
     int colonPos = -1;
     int equalPos = -1;
-    for (int i = 0; i < line.count(); ++i) {
+    for (int i = 0; i < line.size(); ++i) {
         const QChar c = s.at(i);
         if (firstPos < 0 && !c.isSpace()) {
             firstPos = i;
@@ -282,10 +272,10 @@ CMakeConfigItem CMakeConfigItem::fromString(const QString &s)
 
 static QByteArray trimCMakeCacheLine(const QByteArray &in) {
     int start = 0;
-    while (start < in.count() && (in.at(start) == ' ' || in.at(start) == '\t'))
+    while (start < in.size() && (in.at(start) == ' ' || in.at(start) == '\t'))
         ++start;
 
-    return in.mid(start, in.count() - start - 1);
+    return in.mid(start, in.size() - start - 1);
 }
 
 static QByteArrayList splitCMakeCacheLine(const QByteArray &line) {
@@ -359,8 +349,7 @@ CMakeConfig CMakeConfig::fromFile(const Utils::FilePath &cacheFile, QString *err
     QFile cache(cacheFile.toString());
     if (!cache.open(QIODevice::ReadOnly | QIODevice::Text)) {
         if (errorMessage)
-            *errorMessage = QCoreApplication::translate("CMakeProjectManager::CMakeConfigItem", "Failed to open %1 for reading.")
-                .arg(cacheFile.toUserOutput());
+            *errorMessage = Tr::tr("Failed to open %1 for reading.").arg(cacheFile.toUserOutput());
         return CMakeConfig();
     }
 
@@ -382,15 +371,15 @@ CMakeConfig CMakeConfig::fromFile(const Utils::FilePath &cacheFile, QString *err
         if (pieces.isEmpty())
             continue;
 
-        QTC_ASSERT(pieces.count() == 3, continue);
-        const QByteArray key = pieces.at(0);
-        const QByteArray type = pieces.at(1);
-        const QByteArray value = pieces.at(2);
+        QTC_ASSERT(pieces.size() == 3, continue);
+        const QByteArray &key = pieces.at(0);
+        const QByteArray &type = pieces.at(1);
+        const QByteArray &value = pieces.at(2);
 
         if (key.endsWith("-ADVANCED") && value == "1") {
-            advancedSet.insert(key.left(key.count() - 9 /* "-ADVANCED" */));
+            advancedSet.insert(key.left(key.size() - 9 /* "-ADVANCED" */));
         } else if (key.endsWith("-STRINGS") && CMakeConfigItem::typeStringToType(type) == CMakeConfigItem::INTERNAL) {
-            valuesMap[key.left(key.count() - 8) /* "-STRINGS" */] = value;
+            valuesMap[key.left(key.size() - 8) /* "-STRINGS" */] = value;
         } else {
             CMakeConfigItem::Type t = CMakeConfigItem::typeStringToType(type);
             result << CMakeConfigItem(key, t, documentation, value);
@@ -398,7 +387,7 @@ CMakeConfig CMakeConfig::fromFile(const Utils::FilePath &cacheFile, QString *err
     }
 
     // Set advanced flags:
-    for (int i = 0; i < result.count(); ++i) {
+    for (int i = 0; i < result.size(); ++i) {
         CMakeConfigItem &item = result[i];
         item.isAdvanced = advancedSet.contains(item.key);
 
@@ -410,9 +399,7 @@ CMakeConfig CMakeConfig::fromFile(const Utils::FilePath &cacheFile, QString *err
         }
     }
 
-    Utils::sort(result, &CMakeConfigItem::less);
-
-    return result;
+    return Utils::sorted(std::move(result), &CMakeConfigItem::less);
 }
 
 QString CMakeConfigItem::toString(const Utils::MacroExpander *expander) const
@@ -447,9 +434,7 @@ QString CMakeConfigItem::toString(const Utils::MacroExpander *expander) const
         break;
     }
 
-    const QString expandedValue
-            = expander ? expander->expand(QString::fromUtf8(value)) : QString::fromUtf8(value);
-    return QString::fromUtf8(key) + QLatin1Char(':') + typeStr + QLatin1Char('=') + expandedValue;
+    return QString("%1:%2=%3").arg(QString::fromUtf8(key), typeStr, expandedValue(expander));
 }
 
 QString CMakeConfigItem::toArgument() const
@@ -482,7 +467,7 @@ bool CMakeConfigItem::operator==(const CMakeConfigItem &o) const
     return o.key == key && o.value == value && o.isUnset == isUnset && o.isInitial == isInitial;
 }
 
-Utils::QHashValueType qHash(const CMakeConfigItem &it)
+size_t qHash(const CMakeConfigItem &it)
 {
     return ::qHash(it.key) ^ ::qHash(it.value) ^ ::qHash(it.isUnset) ^ ::qHash(it.isInitial);
 }
