@@ -1,46 +1,25 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "lldbengine.h"
 
+#include <debugger/breakhandler.h>
 #include <debugger/debuggeractions.h>
 #include <debugger/debuggercore.h>
 #include <debugger/debuggerdialogs.h>
 #include <debugger/debuggerinternalconstants.h>
+#include <debugger/debuggeritem.h>
 #include <debugger/debuggermainwindow.h>
 #include <debugger/debuggerprotocol.h>
-#include <debugger/debuggertooltipmanager.h>
-#include <debugger/terminal.h>
-
-#include <debugger/breakhandler.h>
 #include <debugger/debuggersourcepathmappingwidget.h>
+#include <debugger/debuggertooltipmanager.h>
+#include <debugger/debuggertr.h>
 #include <debugger/disassemblerlines.h>
 #include <debugger/moduleshandler.h>
 #include <debugger/registerhandler.h>
-#include <debugger/stackhandler.h>
 #include <debugger/sourceutils.h>
+#include <debugger/stackhandler.h>
+#include <debugger/terminal.h>
 #include <debugger/threadshandler.h>
 #include <debugger/watchhandler.h>
 #include <debugger/watchutils.h>
@@ -49,14 +28,14 @@
 #include <coreplugin/idocument.h>
 #include <coreplugin/icore.h>
 
+#include <utils/environment.h>
+#include <utils/process.h>
+#include <utils/processinterface.h>
 #include <utils/qtcassert.h>
-#include <utils/savedaction.h>
-#include <utils/qtcprocess.h>
 
 #include <QApplication>
 #include <QDateTime>
 #include <QDebug>
-#include <QDir>
 #include <QFileInfo>
 #include <QTimer>
 #include <QToolTip>
@@ -67,8 +46,7 @@
 using namespace Core;
 using namespace Utils;
 
-namespace Debugger {
-namespace Internal {
+namespace Debugger::Internal {
 
 static int &currentToken()
 {
@@ -84,38 +62,31 @@ static int &currentToken()
 
 LldbEngine::LldbEngine()
 {
+    m_lldbProc.setUseCtrlCStub(true);
+    m_lldbProc.setProcessMode(ProcessMode::Writer);
+
     setObjectName("LldbEngine");
     setDebuggerName("LLDB");
 
-    connect(action(AutoDerefPointers), &SavedAction::valueChanged,
-            this, &LldbEngine::updateLocals);
-    connect(action(CreateFullBacktrace), &QAction::triggered,
+    DebuggerSettings &ds = settings();
+    connect(&ds.autoDerefPointers, &BaseAspect::changed, this, &LldbEngine::updateLocals);
+    connect(ds.createFullBacktrace.action(), &QAction::triggered,
             this, &LldbEngine::fetchFullBacktrace);
-    connect(action(UseDebuggingHelpers), &SavedAction::valueChanged,
-            this, &LldbEngine::updateLocals);
-    connect(action(UseDynamicType), &SavedAction::valueChanged,
-            this, &LldbEngine::updateLocals);
-    connect(action(IntelFlavor), &SavedAction::valueChanged,
-            this, &LldbEngine::updateAll);
+    connect(&ds.useDebuggingHelpers, &BaseAspect::changed, this, &LldbEngine::updateLocals);
+    connect(&ds.useDynamicType, &BaseAspect::changed, this, &LldbEngine::updateLocals);
+    connect(&ds.intelFlavor, &BaseAspect::changed, this, &LldbEngine::updateAll);
 
-    connect(&m_lldbProc, &QProcess::errorOccurred,
-            this, &LldbEngine::handleLldbError);
-    connect(&m_lldbProc, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
-            this, &LldbEngine::handleLldbFinished);
-    connect(&m_lldbProc, &QProcess::readyReadStandardOutput,
+    connect(&m_lldbProc, &Process::started, this, &LldbEngine::handleLldbStarted);
+    connect(&m_lldbProc, &Process::done, this, &LldbEngine::handleLldbDone);
+    connect(&m_lldbProc, &Process::readyReadStandardOutput,
             this, &LldbEngine::readLldbStandardOutput);
-    connect(&m_lldbProc, &QProcess::readyReadStandardError,
+    connect(&m_lldbProc, &Process::readyReadStandardError,
             this, &LldbEngine::readLldbStandardError);
 
     connect(this, &LldbEngine::outputReady,
             this, &LldbEngine::handleResponse, Qt::QueuedConnection);
 }
 
-LldbEngine::~LldbEngine()
-{
-    m_lldbProc.disconnect();
-}
-
 void LldbEngine::executeDebuggerCommand(const QString &command)
 {
     DebuggerCommand cmd("executeDebuggerCommand");
@@ -125,7 +96,7 @@ void LldbEngine::executeDebuggerCommand(const QString &command)
 
 void LldbEngine::runCommand(const DebuggerCommand &command)
 {
-    if (m_lldbProc.state() != QProcess::Running) {
+    if (!m_lldbProc.isRunning()) {
         // This can legally happen e.g. through a reloadModule()
         // triggered by changes in view visibility.
         showMessage(QString("NO LLDB PROCESS RUNNING, CMD IGNORED: %1 %2")
@@ -143,7 +114,7 @@ void LldbEngine::runCommand(const DebuggerCommand &command)
     if (cmd.flags == NeedsFullStop) {
         cmd.flags &= ~NeedsFullStop;
         if (state() == InferiorRunOk) {
-            showStatusMessage(tr("Stopping temporarily"), 1000);
+            showStatusMessage(Tr::tr("Stopping temporarily"), 1000);
             m_onStop.append(cmd, false);
             interruptInferior();
             return;
@@ -151,7 +122,7 @@ void LldbEngine::runCommand(const DebuggerCommand &command)
     }
     showMessage(msg, LogInput);
     m_commandForToken[currentToken()] = cmd;
-    m_lldbProc.write("script theDumper." + function.toUtf8() + "\n");
+    executeCommand("script theDumper." + function);
 }
 
 void LldbEngine::debugLastCommand()
@@ -168,6 +139,13 @@ void LldbEngine::handleAttachedToCore()
     updateLocals();
 }
 
+void LldbEngine::executeCommand(const QString &command)
+{
+    // For some reason, sometimes LLDB misses the first character of the next command on Windows
+    // if passing only 1 LF.
+    m_lldbProc.write(command + "\n\n");
+}
+
 void LldbEngine::shutdownInferior()
 {
     QTC_ASSERT(state() == InferiorShutdownRequested, qDebug() << state());
@@ -177,74 +155,87 @@ void LldbEngine::shutdownInferior()
 void LldbEngine::shutdownEngine()
 {
     QTC_ASSERT(state() == EngineShutdownRequested, qDebug() << state());
-    if (m_lldbProc.state() == QProcess::Running)
-        m_lldbProc.terminate();
-    else
-        notifyEngineShutdownFinished();
+    abortDebuggerProcess();
 }
 
 void LldbEngine::abortDebuggerProcess()
 {
-    if (m_lldbProc.state() == QProcess::Running)
-        m_lldbProc.kill();
+    if (m_lldbProc.isRunning())
+        m_lldbProc.stop();
     else
         notifyEngineShutdownFinished();
 }
 
 static QString adapterStartFailed()
 {
-    return LldbEngine::tr("Adapter start failed.");
+    return Tr::tr("Adapter start failed.");
 }
 
 void LldbEngine::setupEngine()
 {
     QTC_ASSERT(state() == EngineSetupRequested, qDebug() << state());
 
-    const FilePath lldbCmd = runParameters().debugger.executable;
+    const FilePath lldbCmd = runParameters().debugger.command.executable();
 
     showMessage("STARTING LLDB: " + lldbCmd.toUserOutput());
     Environment environment = runParameters().debugger.environment;
+    environment.appendOrSet("QT_CREATOR_LLDB_PROCESS", "1");
     environment.appendOrSet("PYTHONUNBUFFERED", "1");  // avoid flushing problem on macOS
+    DebuggerItem::addAndroidLldbPythonEnv(lldbCmd, environment);
+
+    if (lldbCmd.osType() == OsTypeLinux) {
+        // LLDB 14 installation on Ubuntu 22.04 is broken:
+        // https://bugs.launchpad.net/ubuntu/+source/llvm-defaults/+bug/1972855
+        // Brush over it:
+        Process lldbPythonPathFinder;
+        lldbPythonPathFinder.setCommand({lldbCmd, {"-P"}});
+        lldbPythonPathFinder.start();
+        lldbPythonPathFinder.waitForFinished();
+        QString lldbPythonPath = lldbPythonPathFinder.cleanedStdOut();
+        if (lldbPythonPath.endsWith('\n'))
+            lldbPythonPath.chop(1);
+        if (lldbPythonPath == "/usr/lib/local/lib/python3.10/dist-packages")
+            environment.appendOrSet("PYTHONPATH", "/usr/lib/llvm-14/lib/python3.10/dist-packages");
+    }
+
     m_lldbProc.setEnvironment(environment);
-    if (QFileInfo(runParameters().debugger.workingDirectory).isDir())
+
+    if (runParameters().debugger.workingDirectory.isDir())
         m_lldbProc.setWorkingDirectory(runParameters().debugger.workingDirectory);
 
-    m_lldbProc.setCommand(CommandLine(lldbCmd));
-    m_lldbProc.start();
+    if (HostOsInfo::isRunningUnderRosetta())
+        m_lldbProc.setCommand(CommandLine("/usr/bin/arch", {"-arm64", lldbCmd.toString()}));
+    else
+        m_lldbProc.setCommand(CommandLine(lldbCmd));
 
-    if (!m_lldbProc.waitForStarted()) {
-        const QString msg = tr("Unable to start LLDB \"%1\": %2")
-            .arg(lldbCmd.toUserOutput(), m_lldbProc.errorString());
-        notifyEngineSetupFailed();
-        showMessage("ADAPTER START FAILED");
-        if (!msg.isEmpty())
-            ICore::showWarningWithOptions(adapterStartFailed(), msg);
-        return;
-    }
+    m_lldbProc.start();
+}
+
+void LldbEngine::handleLldbStarted()
+{
     m_lldbProc.waitForReadyRead(1000);
 
-    showStatusMessage(tr("Setting up inferior..."));
+    showStatusMessage(Tr::tr("Setting up inferior..."));
 
-    const QByteArray dumperSourcePath =
-        ICore::resourcePath().toLocal8Bit() + "/debugger/";
+    const DebuggerRunParameters &rp = runParameters();
 
-    m_lldbProc.write("script sys.path.insert(1, '" + dumperSourcePath + "')\n");
+    QString dumperPath = ICore::resourcePath("debugger").path();
+    executeCommand("script sys.path.insert(1, '" + dumperPath + "')");
     // This triggers reportState("enginesetupok") or "enginesetupfailed":
-    m_lldbProc.write("script from lldbbridge import *\n");
+    executeCommand("script from lldbbridge import *");
 
     QString commands = nativeStartupCommands();
     if (!commands.isEmpty())
-        m_lldbProc.write(commands.toLocal8Bit() + '\n');
+        executeCommand(commands);
 
-
-    const QString path = stringSetting(ExtraDumperFile);
-    if (!path.isEmpty() && QFileInfo(path).isReadable()) {
+    const FilePath path = settings().extraDumperFile();
+    if (!path.isEmpty() && path.isReadableFile()) {
         DebuggerCommand cmd("addDumperModule");
-        cmd.arg("path", path);
+        cmd.arg("path", path.path());
         runCommand(cmd);
     }
 
-    commands = stringSetting(ExtraDumperCommands);
+    commands = settings().extraDumperCommands();
     if (!commands.isEmpty()) {
         DebuggerCommand cmd("executeDebuggerCommand");
         cmd.arg("command", commands);
@@ -257,11 +248,7 @@ void LldbEngine::setupEngine()
     };
     runCommand(cmd1);
 
-    const DebuggerRunParameters &rp = runParameters();
-
-    const SourcePathMap sourcePathMap =
-        DebuggerSourcePathMappingWidget::mergePlatformQtPath(rp,
-                Internal::globalDebuggerOptions()->sourcePathMap);
+    const SourcePathMap sourcePathMap = mergePlatformQtPath(rp, settings().sourcePathMap());
     for (auto it = sourcePathMap.constBegin(), cend = sourcePathMap.constEnd();
          it != cend;
          ++it) {
@@ -269,17 +256,21 @@ void LldbEngine::setupEngine()
                     "settings append target.source-map " + it.key() + ' ' + expand(it.value()));
     }
 
+    for (const FilePath &path : rp.solibSearchPath)
+        executeDebuggerCommand("settings append target.exec-search-paths " + path.toString());
+
     DebuggerCommand cmd2("setupInferior");
-    cmd2.arg("executable", rp.inferior.executable.toString());
+    cmd2.arg("executable", rp.inferior.command.executable().path());
     cmd2.arg("breakonmain", rp.breakOnMain);
     cmd2.arg("useterminal", bool(terminal()));
     cmd2.arg("startmode", rp.startMode);
     cmd2.arg("nativemixed", isNativeMixedActive());
-    cmd2.arg("workingdirectory", rp.inferior.workingDirectory);
+    cmd2.arg("workingdirectory", rp.inferior.workingDirectory.path());
     cmd2.arg("environment", rp.inferior.environment.toStringList());
-    cmd2.arg("processargs", toHex(QtcProcess::splitArgs(rp.inferior.commandLineArguments).join(QChar(0))));
+    cmd2.arg("processargs", toHex(ProcessArgs::splitArgs(rp.inferior.command.arguments(),
+                                                         HostOsInfo::hostOs()).join(QChar(0))));
     cmd2.arg("platform", rp.platform);
-    cmd2.arg("symbolfile", rp.symbolFile);
+    cmd2.arg("symbolfile", rp.symbolFile.path());
 
     if (terminal()) {
         const qint64 attachedPID = terminal()->applicationPid();
@@ -288,24 +279,23 @@ void LldbEngine::setupEngine()
                 ? QString::fromLatin1("Attaching to %1 (%2)").arg(attachedPID).arg(attachedMainThreadID)
                 : QString::fromLatin1("Attaching to %1").arg(attachedPID);
         showMessage(msg, LogMisc);
+        cmd2.arg("startmode", DebuggerStartMode::AttachToLocalProcess);
         cmd2.arg("attachpid", attachedPID);
-
     } else {
-
         cmd2.arg("startmode", rp.startMode);
         // it is better not to check the start mode on the python sid (as we would have to duplicate the
         // enum values), and thus we assume that if the rp.attachPID is valid we really have to attach
-        QTC_CHECK(!rp.attachPID.isValid() || (rp.startMode == AttachCrashedExternal
-                                              || rp.startMode == AttachExternal));
+        QTC_CHECK(rp.attachPID.isValid() && (rp.startMode == AttachToRemoteProcess
+                                             || rp.startMode == AttachToLocalProcess
+                                             || rp.startMode == AttachToRemoteServer));
         cmd2.arg("attachpid", rp.attachPID.pid());
         cmd2.arg("sysroot", rp.deviceSymbolsRoot.isEmpty() ? rp.sysRoot.toString()
                                                            : rp.deviceSymbolsRoot);
         cmd2.arg("remotechannel", ((rp.startMode == AttachToRemoteProcess
                                    || rp.startMode == AttachToRemoteServer)
                                   ? rp.remoteChannel : QString()));
-        cmd2.arg("platform", rp.platform);
         QTC_CHECK(!rp.continueAfterAttach || (rp.startMode == AttachToRemoteProcess
-                                              || rp.startMode == AttachExternal
+                                              || rp.startMode == AttachToLocalProcess
                                               || rp.startMode == AttachToRemoteServer));
         m_continueAtNextSpontaneousStop = false;
     }
@@ -317,7 +307,10 @@ void LldbEngine::setupEngine()
             // Some extra roundtrip to make sure we end up behind all commands triggered
             // from claimBreakpointsForEngine().
             DebuggerCommand cmd3("executeRoundtrip");
-            cmd3.callback = [this](const DebuggerResponse &) { notifyEngineSetupOk(); };
+            cmd3.callback = [this](const DebuggerResponse &) {
+                notifyEngineSetupOk();
+                runEngine();
+            };
             runCommand(cmd3);
         } else {
             notifyEngineSetupFailed();
@@ -326,22 +319,26 @@ void LldbEngine::setupEngine()
 
     cmd2.flags = Silent;
     runCommand(cmd2);
+
+    DebuggerCommand cmd0("setFallbackQtVersion");
+    cmd0.arg("version", rp.fallbackQtVersion);
+    runCommand(cmd0);
 }
 
 void LldbEngine::runEngine()
 {
     const DebuggerRunParameters &rp = runParameters();
     QTC_ASSERT(state() == EngineRunRequested, qDebug() << state(); return);
-    showStatusMessage(tr("Running requested..."), 5000);
+    showStatusMessage(Tr::tr("Running requested..."), 5000);
     DebuggerCommand cmd("runEngine");
-    if (rp.startMode == AttachCore)
-        cmd.arg("coreFile", rp.coreFile);
+    if (rp.startMode == AttachToCore)
+        cmd.arg("coreFile", rp.coreFile.path());
     runCommand(cmd);
 }
 
 void LldbEngine::interruptInferior()
 {
-    showStatusMessage(tr("Interrupt requested..."), 5000);
+    showStatusMessage(Tr::tr("Interrupt requested..."), 5000);
     runCommand({"interruptInferior"});
 }
 
@@ -416,8 +413,8 @@ void LldbEngine::executeRunToLine(const ContextData &data)
 {
     notifyInferiorRunRequested();
     DebuggerCommand cmd("executeRunToLocation");
-    cmd.arg("file", data.fileName);
-    cmd.arg("line", data.lineNumber);
+    cmd.arg("file", data.fileName.path());
+    cmd.arg("line", data.textPosition.line);
     cmd.arg("address", data.address);
     runCommand(cmd);
 }
@@ -433,8 +430,8 @@ void LldbEngine::executeRunToFunction(const QString &functionName)
 void LldbEngine::executeJumpToLine(const ContextData &data)
 {
     DebuggerCommand cmd("executeJumpToLocation");
-    cmd.arg("file", data.fileName);
-    cmd.arg("line", data.lineNumber);
+    cmd.arg("file", data.fileName.path());
+    cmd.arg("line", data.textPosition.line);
     cmd.arg("address", data.address);
     runCommand(cmd);
 }
@@ -470,28 +467,14 @@ void LldbEngine::selectThread(const Thread &thread)
     DebuggerCommand cmd("selectThread");
     cmd.arg("id", thread->id());
     cmd.callback = [this](const DebuggerResponse &) {
-        fetchStack(action(MaximalStackDepth)->value().toInt());
+        fetchStack(settings().maximalStackDepth());
     };
     runCommand(cmd);
 }
 
-bool LldbEngine::stateAcceptsBreakpointChanges() const
-{
-    switch (state()) {
-    case EngineSetupRequested:
-    case InferiorRunRequested:
-    case InferiorRunOk:
-    case InferiorStopRequested:
-    case InferiorStopOk:
-        return true;
-    default:
-        return false;
-    }
-}
-
 bool LldbEngine::acceptsBreakpoint(const BreakpointParameters &bp) const
 {
-    if (runParameters().startMode == AttachCore)
+    if (runParameters().startMode == AttachToCore)
         return false;
     if (bp.isCppBreakpoint())
         return true;
@@ -576,7 +559,7 @@ void LldbEngine::updateBreakpointData(const Breakpoint &bp, const GdbMi &bkpt, b
     bp->setCondition(fromHex(bkpt["condition"].data()));
     bp->setHitCount(bkpt["hitcount"].toInt());
     bp->setFileName(FilePath::fromUserInput(bkpt["file"].data()));
-    bp->setLineNumber(bkpt["line"].toInt());
+    bp->setTextPosition({bkpt["line"].toInt(), -1});
 
     GdbMi locations = bkpt["locations"];
     const int numChild = locations.childCount();
@@ -589,7 +572,7 @@ void LldbEngine::updateBreakpointData(const Breakpoint &bp, const GdbMi &bkpt, b
             loc->params.address = location["addr"].toAddress();
             loc->params.functionName = location["function"].data();
             loc->params.fileName = FilePath::fromUserInput(location["file"].data());
-            loc->params.lineNumber = location["line"].toInt();
+            loc->params.textPosition.line = location["line"].toInt();
             loc->displayName = QString("%1.%2").arg(bp->responseId()).arg(locid);
         }
         bp->setPending(false);
@@ -646,7 +629,7 @@ void LldbEngine::handleInterpreterBreakpointModified(const GdbMi &bpItem)
     updateBreakpointData(bp, bpItem, false);
 }
 
-void LldbEngine::loadSymbols(const QString &moduleName)
+void LldbEngine::loadSymbols(const FilePath &moduleName)
 {
     Q_UNUSED(moduleName)
 }
@@ -659,12 +642,13 @@ void LldbEngine::reloadModules()
 {
     DebuggerCommand cmd("fetchModules");
     cmd.callback = [this](const DebuggerResponse &response) {
+        const FilePath inferior = runParameters().inferior.command.executable();
         const GdbMi &modules = response.data["modules"];
         ModulesHandler *handler = modulesHandler();
         handler->beginUpdateAll();
         for (const GdbMi &item : modules) {
             Module module;
-            module.modulePath = item["file"].data();
+            module.modulePath = inferior.withNewPath(item["file"].data());
             module.moduleName = item["name"].data();
             module.symbolsRead = Module::UnknownReadState;
             module.startAddress = item["loaded_addr"].toAddress();
@@ -676,13 +660,13 @@ void LldbEngine::reloadModules()
     runCommand(cmd);
 }
 
-void LldbEngine::requestModuleSymbols(const QString &moduleName)
+void LldbEngine::requestModuleSymbols(const FilePath &moduleName)
 {
     DebuggerCommand cmd("fetchSymbols");
-    cmd.arg("module", moduleName);
+    cmd.arg("module", moduleName.path());
     cmd.callback = [moduleName](const DebuggerResponse &response) {
         const GdbMi &symbols = response.data["symbols"];
-        QString moduleName = response.data["module"].data();
+        const QString module = response.data["module"].data();
         Symbols syms;
         for (const GdbMi &item : symbols) {
             Symbol symbol;
@@ -693,7 +677,7 @@ void LldbEngine::requestModuleSymbols(const QString &moduleName)
             symbol.demangled = item["demangled"].data();
             syms.append(symbol);
         }
-        showModuleSymbols(moduleName, syms);
+        showModuleSymbols(moduleName.withNewPath(module), syms);
     };
     runCommand(cmd);
 }
@@ -715,33 +699,12 @@ void LldbEngine::updateAll()
     DebuggerCommand cmd("fetchThreads");
     cmd.callback = [this](const DebuggerResponse &response) {
         threadsHandler()->setThreads(response.data);
-        fetchStack(action(MaximalStackDepth)->value().toInt());
+        fetchStack(settings().maximalStackDepth());
         reloadRegisters();
     };
     runCommand(cmd);
 }
 
-void LldbEngine::reloadFullStack()
-{
-    fetchStack(-1);
-}
-
-void LldbEngine::fetchStack(int limit)
-{
-    DebuggerCommand cmd("fetchStack");
-    cmd.arg("nativemixed", isNativeMixedActive());
-    cmd.arg("stacklimit", limit);
-    cmd.arg("context", stackHandler()->currentFrame().context);
-    cmd.callback = [this](const DebuggerResponse &response) {
-        const GdbMi &stack = response.data["stack"];
-        const bool isFull = !stack["hasmore"].toInt();
-        stackHandler()->setFramesAndCurrentIndex(stack["frames"], isFull);
-        activateFrame(stackHandler()->currentIndex());
-    };
-    runCommand(cmd);
-}
-
-
 //////////////////////////////////////////////////////////////////////
 //
 // Watch specific stuff
@@ -768,21 +731,22 @@ void LldbEngine::doUpdateLocals(const UpdateParameters &params)
     watchHandler()->appendFormatRequests(&cmd);
     watchHandler()->appendWatchersAndTooltipRequests(&cmd);
 
-    const static bool alwaysVerbose = qEnvironmentVariableIsSet("QTC_DEBUGGER_PYTHON_VERBOSE");
+    const bool alwaysVerbose = qtcEnvironmentVariableIsSet("QTC_DEBUGGER_PYTHON_VERBOSE");
+    const DebuggerSettings &s = settings();
     cmd.arg("passexceptions", alwaysVerbose);
-    cmd.arg("fancy", boolSetting(UseDebuggingHelpers));
-    cmd.arg("autoderef", boolSetting(AutoDerefPointers));
-    cmd.arg("dyntype", boolSetting(UseDynamicType));
+    cmd.arg("fancy", s.useDebuggingHelpers());
+    cmd.arg("autoderef", s.autoDerefPointers());
+    cmd.arg("dyntype", s.useDynamicType());
     cmd.arg("partialvar", params.partialVariable);
-    cmd.arg("qobjectnames", boolSetting(ShowQObjectNames));
-    cmd.arg("timestamps", boolSetting(LogTimeStamps));
+    cmd.arg("qobjectnames", s.showQObjectNames());
+    cmd.arg("timestamps", s.logTimeStamps());
 
     StackFrame frame = stackHandler()->currentFrame();
     cmd.arg("context", frame.context);
     cmd.arg("nativemixed", isNativeMixedActive());
 
-    cmd.arg("stringcutoff", action(MaximalStringLength)->value().toString());
-    cmd.arg("displaystringlimit", action(DisplayStringLimit)->value().toString());
+    cmd.arg("stringcutoff", s.maximalStringLength());
+    cmd.arg("displaystringlimit", s.displayStringLimit());
 
     //cmd.arg("resultvarname", m_resultVarName);
     cmd.arg("partialvar", params.partialVariable);
@@ -799,12 +763,26 @@ void LldbEngine::doUpdateLocals(const UpdateParameters &params)
     runCommand(cmd);
 }
 
-void LldbEngine::handleLldbError(QProcess::ProcessError error)
+void LldbEngine::handleLldbDone()
 {
+    if (m_lldbProc.result() == ProcessResult::StartFailed) {
+        notifyEngineSetupFailed();
+        showMessage("ADAPTER START FAILED");
+        ICore::showWarningWithOptions(adapterStartFailed(), Tr::tr("Unable to start LLDB \"%1\": %2")
+                 .arg(runParameters().debugger.command.executable().toUserOutput(),
+                 m_lldbProc.errorString()));
+        return;
+    }
+
+    if (m_lldbProc.error() == QProcess::UnknownError) {
+        notifyDebuggerProcessFinished(m_lldbProc.resultData(), "LLDB");
+        return;
+    }
+
+    const QProcess::ProcessError error = m_lldbProc.error();
     showMessage(QString("LLDB PROCESS ERROR: %1").arg(error));
     switch (error) {
     case QProcess::Crashed:
-        m_lldbProc.disconnect();
         notifyEngineShutdownFinished();
         break; // will get a processExited() as well
     // impossible case QProcess::FailedToStart:
@@ -813,8 +791,7 @@ void LldbEngine::handleLldbError(QProcess::ProcessError error)
     case QProcess::Timedout:
     default:
         //setState(EngineShutdownRequested, true);
-        m_lldbProc.kill();
-        AsynchronousMessageBox::critical(tr("LLDB I/O Error"), errorMessage(error));
+        AsynchronousMessageBox::critical(Tr::tr("LLDB I/O Error"), errorMessage(error));
         break;
     }
 }
@@ -823,44 +800,39 @@ QString LldbEngine::errorMessage(QProcess::ProcessError error) const
 {
     switch (error) {
         case QProcess::FailedToStart:
-            return tr("The LLDB process failed to start. Either the "
+            return Tr::tr("The LLDB process failed to start. Either the "
                 "invoked program \"%1\" is missing, or you may have insufficient "
                 "permissions to invoke the program.")
-                .arg(runParameters().debugger.executable.toUserOutput());
+                .arg(runParameters().debugger.command.executable().toUserOutput());
         case QProcess::Crashed:
-            return tr("The LLDB process crashed some time after starting "
+            return Tr::tr("The LLDB process crashed some time after starting "
                 "successfully.");
         case QProcess::Timedout:
-            return tr("The last waitFor...() function timed out. "
+            return Tr::tr("The last waitFor...() function timed out. "
                 "The state of QProcess is unchanged, and you can try calling "
                 "waitFor...() again.");
         case QProcess::WriteError:
-            return tr("An error occurred when attempting to write "
+            return Tr::tr("An error occurred when attempting to write "
                 "to the LLDB process. For example, the process may not be running, "
                 "or it may have closed its input channel.");
         case QProcess::ReadError:
-            return tr("An error occurred when attempting to read from "
+            return Tr::tr("An error occurred when attempting to read from "
                 "the Lldb process. For example, the process may not be running.");
         default:
-            return tr("An unknown error in the LLDB process occurred.") + ' ';
+            return Tr::tr("An unknown error in the LLDB process occurred.") + ' ';
     }
 }
 
-void LldbEngine::handleLldbFinished(int exitCode, QProcess::ExitStatus exitStatus)
-{
-    notifyDebuggerProcessFinished(exitCode, exitStatus, "LLDB");
-}
-
 void LldbEngine::readLldbStandardError()
 {
-    QString err = QString::fromUtf8(m_lldbProc.readAllStandardError());
+    QString err = QString::fromUtf8(m_lldbProc.readAllRawStandardError());
     qDebug() << "\nLLDB STDERR UNEXPECTED: " << err;
     showMessage("Lldb stderr: " + err, LogError);
 }
 
 void LldbEngine::readLldbStandardOutput()
 {
-    QByteArray outba = m_lldbProc.readAllStandardOutput();
+    QByteArray outba = m_lldbProc.readAllRawStandardOutput();
     outba.replace("\r\n", "\n");
     QString out = QString::fromUtf8(outba);
     showMessage(out, LogOutput);
@@ -925,7 +897,7 @@ void LldbEngine::handleStateNotification(const GdbMi &item)
         continueInferior();
     } else if (newState == "enginerunokandinferiorunrunnable") {
         notifyEngineRunOkAndInferiorUnrunnable();
-        if (runParameters().startMode == AttachCore)
+        if (runParameters().startMode == AttachToCore)
             handleAttachedToCore();
     } else if (newState == "inferiorshutdownfinished")
         notifyInferiorShutdownFinished();
@@ -938,7 +910,7 @@ void LldbEngine::handleStateNotification(const GdbMi &item)
 void LldbEngine::handleLocationNotification(const GdbMi &reportedLocation)
 {
     qulonglong address = reportedLocation["address"].toAddress();
-    Utils::FilePath fileName = FilePath::fromUserInput(reportedLocation["file"].data());
+    FilePath fileName = FilePath::fromUserInput(reportedLocation["file"].data());
     QString function = reportedLocation["function"].data();
     int lineNumber = reportedLocation["line"].toInt();
     Location loc = Location(fileName, lineNumber);
@@ -985,6 +957,33 @@ void LldbEngine::reloadDebuggingHelpers()
     updateAll();
 }
 
+void LldbEngine::loadAdditionalQmlStack()
+{
+    fetchStack(-1, true);
+}
+
+void LldbEngine::reloadFullStack()
+{
+    fetchStack(-1, false);
+}
+
+void LldbEngine::fetchStack(int limit, bool extraQml)
+{
+    DebuggerCommand cmd("fetchStack");
+    cmd.arg("nativemixed", isNativeMixedActive());
+    cmd.arg("stacklimit", limit);
+    cmd.arg("context", stackHandler()->currentFrame().context);
+    cmd.arg("extraqml", int(extraQml));
+    cmd.callback = [this](const DebuggerResponse &response) {
+        const GdbMi &stack = response.data["stack"];
+        const bool isFull = !stack["hasmore"].toInt();
+        stackHandler()->setFramesAndCurrentIndex(stack["frames"], isFull);
+        activateFrame(stackHandler()->currentIndex());
+    };
+    runCommand(cmd);
+}
+
+
 void LldbEngine::fetchDisassembler(DisassemblerAgent *agent)
 {
     QPointer<DisassemblerAgent> p(agent);
@@ -997,7 +996,7 @@ void LldbEngine::fetchDisassembler(DisassemblerAgent *agent)
     DebuggerCommand cmd("fetchDisassembler");
     cmd.arg("address", loc.address());
     cmd.arg("function", loc.functionName());
-    cmd.arg("flavor", boolSetting(IntelFlavor) ? "intel" : "att");
+    cmd.arg("flavor", settings().intelFlavor() ? "intel" : "att");
     cmd.callback = [this, id](const DebuggerResponse &response) {
         DisassemblerLines result;
         QPointer<DisassemblerAgent> agent = m_disassemblerAgents.key(id);
@@ -1031,8 +1030,8 @@ void LldbEngine::fetchDisassembler(DisassemblerAgent *agent)
 void LldbEngine::fetchFullBacktrace()
 {
     DebuggerCommand cmd("fetchFullBacktrace");
-    cmd.callback = [](const DebuggerResponse &response)  {
-        Internal::openTextEditor("Backtrace $", fromHex(response.data.data()));
+    cmd.callback = [](const DebuggerResponse &response) {
+        Internal::openTextEditor("Backtrace $", fromHex(response.data["fulltrace"].data()));
     };
     runCommand(cmd);
 }
@@ -1095,10 +1094,11 @@ bool LldbEngine::hasCapability(unsigned cap) const
         | OperateByInstructionCapability
         | RunToLineCapability
         | WatchComplexExpressionsCapability
-        | MemoryAddressCapability))
+        | MemoryAddressCapability
+        | AdditionalQmlStackCapability))
         return true;
 
-    if (runParameters().startMode == AttachCore)
+    if (runParameters().startMode == AttachToCore)
         return false;
 
     //return cap == SnapshotCapability;
@@ -1110,5 +1110,4 @@ DebuggerEngine *createLldbEngine()
     return new LldbEngine;
 }
 
-} // namespace Internal
-} // namespace Debugger
+} // Debugger::Internal
